-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- Freeciv - Copyright (C) 2007 - The Freeciv Project
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2, or (at your option)
--   any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @script I LUV Connections
-- @author LeoPriori
-- @version 1.2.3 (freeciv v3.2+)
-- @description 
-- 
--   Connect your own resources by any roads or river.
-- 
-- @context 
--
--   With this script you can configure your own
--     resources to connect, the units that will reveal
--     the hidden resources, the factories that can 
--     generate new resources and also buildings that 
--     can share resources between cities.
-- 
-- 
-- 
-- @how_can_i_play
--
--    For a resource to be considered 'connected' and capable
--      of sharing with other cities, it is necessary:
--
--   a) To be WITHIN (i.e. INSIDE) the city radius. 
--
--   b) To have an Infrastructure (Mine, Irrigation, etc.).
--
--   c) To have rivers, roads, maglevs or HAVE NONE OF THEM!
--      Some resources require roads to be connected, others do not.
--         If a resource requires roads, there must be some value from 
--         the __roads table between the resource and the city center.
--         If not, just the infrastructure will be enough.
--
--      But if the resource is outside the city borders,
--        and still WITHIN the city radius, such as fish, whales,
--        or a resource located on another continent or on an island,
--        you need to use the following action: 
--      ====> "Connect/Disconnect resource out of border" <====
--
--      Do the following:
--      1) Build the infrastructure on the out-of-reach resource.
--      2) Take any unit with the "Settlers" flag to the CITY CENTER.
--      3) Press "D" (Do action ...) and click in the out-of-reach resource.
--      4) Use the "Connect/Disconnect resource out of border" Action.
--
--      Obs: You can associate this action to any other unit by simply 
--           setting the [actionenabler-*] in the actions.ruleset file.
--
--
--
-- @resources_types
--
--   There are two types of resources: NATURAL and MANUFACTURED.
--
--   Natural resources are those collected on the ground, 
--     while manufactured resources only appear in the city 
--     when a specific building is constructed.
--
--   And some natural resources can be hidden from the player, 
--     being revealed only by the action of some specific units.
--
--
--
-- @export_import_system
--
--   To export a resource, just create a unit in the city 
--     where you already have a connection to that resource 
--     and take it to the desired city.
--   Only one resource per city can be exported. In other words, 
--     the same city cannot export the same resource twice.
--   You can create units to export whatever resources you want 
--     to other cities.
--   This ruleset is set to export only those resources that are 
--     needed to build units or buildings. 
--   Other resources only provide effects and bonuses.
--
--
--
-- @factories
--
--   Factories only generate new resources when the city in which
--     they are built is connected to all the necessary resources, 
--     regardless of whether the resource is imported or shared 
--     from another building such as an Airport.
--
--
--
-- @what_to_do
--   
--   a) Each NATURAL and NOT hidden resource needs:
--     - 1 resource defined as [extra_*] in terrain.ruleset file
--       * <resource> = Resource name (Ex: "Spice", "Wheat", etc.)
--       Obs: There is no need for any [resource_*] tag anymore.
--   
--     - 3 extras defined as [extra_*] in terrain.ruleset file
--       * Connection_<resource> (Ex: "Connection_Gold", "Connection_Coal", etc.)
--       * Connection_<resource>_EXP (to export/import system)
--       * Connection_<resource>_IMP (to export/import system)
--   
--   b) Each NATURAL and hidden resource needs:
--     - 1 resource defined as [extra_*] in terrain.ruleset file
--   
--     - 5 extras defined as [extra_*] in terrain.ruleset file
--       * Connection_<resource>
--       * Connection_<resource>_EXP (to export/import system)
--       * Connection_<resource>_IMP (to export/import system)
--       * Mark_<resource>
--       * Reveal_<resource>
--   
--   c) Each MANUFACTURED resource needs: 
--     - 3 extras defined as [extra_*] in terrain.ruleset file.
--       * Connection_<resource>
--       * Connection_<resource>_EXP (to export/import system)
--       * Connection_<resource>_IMP (to export/import system)
--   
--     - 1 building defined as [building_*] in buildings.ruleset file
--       * Any name
--   
--   d) Each UNIT that can reveal natural resources needs:
--     - 1 unit defined as [unit_*] in units.ruleset file
--       * Any name with the "SearchResources" flag
--   
--   e) Each BUILDING that can share resources needs: 
--     - 1 building defined as [building_*] in buildings.ruleset file
--       * Any name (ex:"Airports", "Harbour", etc)
--   
--   f) Each UNIT that can Export resource needs: 
--     - 3 units defined as [unit_*] in units.ruleset file
--       * Export_<resource> 
--       * Export_Break_<resource> 
--       * Import_Break_<resource> 
--       * "TradeResource" flag for all types above
--       * "BreakExportation" flag only for Break units
--   
--   g) Script Control needs:
--     - 4 extras defined as [extra_*] in terrain.ruleset:
--       * Unreachable_Resource 
--       * SearchResources_BEGIN 
--       * SearchResources_READY 
--       * SearchResources_WORKING 
--   
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- Change the tables below according to your own rules.
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table  __roads
-- 
--   All the available ways to connect.
-- 
-- @values <road>     array       Road name defined as [road_*] in
--                                terrain.ruleset file 
__roads = {
	'River', 'Road', 'Railroad', 'Maglev'
}

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __resources_natural
-- 
--   NATURAL resources and their infrastructures. 
-- 
-- @field resource   string       Resource name, defined as [extra_*] 
--                                in terrain.ruleset file
-- @field tech       string       Advanced needed to reveal and/or access, 
--                                defined as [advance_*] in techs.ruleset file 
-- @field extra      bool         Does this resource need an extra? 
--                                Stones and Wood don't.
-- @field infras     array        All infrastructures that allow to collect this resource.
-- @key <extra>      string       Infrastructure name (ex: Mine, Irrigation, etc),
--                                defined as [extra_*] in terrain.ruleset file
-- @field road       bool         Does this infra need any road to connect? 
--                                Oil Platform in Deep Ocean don't.
-- @field terrains   array        Terrains names this infra can collect, 
--                                defined as [terrain_*] in terrain.ruleset file
-- 
__resources_natural = {
	-- ----------------------------------------------
	{	resource = 'Bauxite',
		tech     = 'Refining',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Hills', 'Mountains'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Cattle',
		tech     = '',
		extra    = true,
		infras   = {
			['Pasture'] = {
				road     = true,
				terrains = {'Grassland'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Clay',
		tech     = '',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Plains', 'Swamp'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Coal',
		tech     = 'Bronze Working',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Hills', 'Mountains'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Corn',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation'] = {
				road     = true,
				terrains = {'Plains'}
			},
			['Farmland'] = {
				road     = true,
				terrains = {'Plains'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Cotton',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation'] = {
				road     = true,
				terrains = {'Grassland', 'Plains'}
			},
			['Farmland'] = {
				road     = true,
				terrains = {'Grassland', 'Plains'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Fish',
		tech     = '',
		extra    = true,
		infras   = {
			['Fishing'] = {
				road     = false,
				terrains = {'Deep Ocean', 'Lake', 'Ocean'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Fruit',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation_Fruit'] = {
				road     = true,
				terrains = {'Jungle'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Furs',
		tech     = '',
		extra    = true,
		infras   = {
			['Pasture'] = {
				road     = true,
				terrains = {'Tundra'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Game',
		tech     = '',
		extra    = true,
		infras   = {
			['Pasture'] = {
				road     = true,
				terrains = {'Tundra'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Gems',
		tech     = '',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Mountains'}
			},
			['Irrigation_Gems'] = {
				road     = true,
				terrains = {'Jungle'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Gold',
		tech     = '',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Hills', 'Mountains'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Horses',
		tech     = '', -- 'Horseback Riding'
		extra    = true,
		infras   = {
			['Pasture'] = {
				road     = true,
				terrains = {'Grassland', 'Plains'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Iron_Ore',
		tech     = 'Iron Working',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Hills', 'Mountains'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Ivory',
		tech     = '',
		extra    = true,
		infras   = {
			['Pasture'] = {
				road     = true,
				terrains = {'Glacier'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Latex',
		tech     = 'Chemistry',
		extra    = true,
		infras   = {
			['Irrigation_Latex'] = {
				road     = true,
				terrains = {'Forest'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Limestone',
		tech     = 'Masonry',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Hills', 'Mountains'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Oasis',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation'] = {
				road     = true,
				terrains = {'Desert'}
			},
			['Farmland'] = {
				road     = true,
				terrains = {'Desert'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Oil',
		tech     = 'Refining',
		extra    = true,
		infras   = {
			['Oil Well'] = {
				road     = true,
				terrains = {'Desert', 'Glacier', 'Tundra'},
			},
			['Oil Platform'] = {
				road     = false,
				terrains = {'Deep Ocean'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Peat',
		tech     = '',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Swamp'},
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Pheasant',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation_Pheasant'] = {
				road     = true,
				terrains = {'Forest'},
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Saltpeter',
		tech     = 'Gunpowder',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Desert', 'Grassland', 'Tundra'},
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Seals',
		tech     = '',
		extra    = true,
		infras   = {
			['Pasture'] = {
				road     = true,
				terrains = {'Glacier'},
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Shellfish',
		tech     = '',
		extra    = true,
		infras   = {
			['Fishing'] = {
				road     = false,
				terrains = {'Lake', 'Ocean', 'Deep Ocean'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Silk',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation_Silk'] = {
				road     = true,
				terrains = {'Forest'},
			},
		}
	},	
	-- ----------------------------------------------
	{	resource = 'Spice',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation'] = {
				road     = true,
				terrains = {'Swamp'},
			},
			['Farmland'] = {
				road     = true,
				terrains = {'Plains'},
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Stones',
		tech     = '',
		extra    = false,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Mountains'}
			},
		}
	},	
	-- ----------------------------------------------
	{	resource = 'Sulfur',
		tech     = 'Gunpowder',
		extra    = true,
		infras   = {
			['Irrigation_Sulfur'] = {
				road     = true,
				terrains = {'Jungle'},
			},
			['Mine'] = {
				road     = true,
				terrains = {'Hills', 'Mountains'}
			},

		}
	},
	-- ----------------------------------------------
	{	resource = 'Tea',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation'] = {
				road     = true,
				terrains = {'Grassland'}
			},
			['Farmland'] = {
				road     = true,
				terrains = {'Grassland'}
			},	
		}
	},
	-- ----------------------------------------------
	{	resource = 'Uranium',
		tech     = 'Atomic Theory',
		extra    = true,
		infras   = {
			['Mine'] = {
				road     = true,
				terrains = {'Mountains', 'Hills'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Whales',
		tech     = '',
		extra    = true,
		infras   = {
			['Fishing'] = {
				road     = false,
				terrains = {'Ocean', 'Deep Ocean'}
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Wheat',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation'] = {
				road     = true,
				terrains = {'Plains'}
			},
			['Farmland'] = {
				road     = true,
				terrains = {'Plains'},
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Wine',
		tech     = '',
		extra    = true,
		infras   = {
			['Irrigation'] = {
				road     = true,
				terrains = {'Plains'}
			},
			['Farmland'] = {
				road     = true,
				terrains = {'Plains'},
			},
		}
	},
	-- ----------------------------------------------
	{	resource = 'Wood',
		tech     = '',
		extra    = false,
		infras   = {
			['Lumber'] = {
				road     = true,
				terrains = {'Forest', 'Jungle'}
			},
		}
	},

}

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __resources_manufactured 
-- 
--   MANUFACTURED resources, their factories and the resources 
--     needed to create it.
-- 
--   IMPORTANT: THE ORDER will ensure that the list is 
--     read in order of dependent factories.
--     Example: The resource 'Coal' must come BEFORE resources
--     that have factories that require 'Coal'.
-- 
-- @field resource   string       Resource name, defined as [extra_*] in 
--                                terrain.ruleset file.
-- @field tech       string       Advanced needed to reveal and/or access.
--                                defined as [advance_*] in techs.ruleset file
--                                Empty values is auto reveal for all
-- @field buildings  array        All the buildings that can create this resource
-- @key   <building> string       Building name 
--                                defined as [building_*] in buildings.ruleset file
-- @field <resource> array       All required resources as defined in one of two tables:
--                                * __resources_natural;
--                                * __resources_manufactured. (in order of dependencies)
__resources_manufactured = {
	-- ----------------------------------------------
	{	resource  = 'Coal',
		tech      = 'Bronze Working',
		buildings = {
			['Factory_Coal'] = {'Stones', 'Wood'},
		},
	},	
	-- ----------------------------------------------
	{	resource  = 'Iron',
		tech      = 'Iron Working',
		buildings = {
			['Factory_Iron'] = {'Coal', 'Iron_Ore'},
		},
	},
	-- ----------------------------------------------
	{	resource  = 'Niter',
		tech      = 'Gunpowder',
		buildings = {
			['Factory_Niter'] = {'Coal', 'Saltpeter', 'Sulfur'},
		},
	},
	-- ----------------------------------------------
	{	resource  = 'Rubber',
		tech      = 'Chemistry',
		buildings = {
			['Factory_Rubber'] = {'Coal', 'Latex', 'Sulfur'},
		},
	},
	-- ----------------------------------------------
	{	resource  = 'Energy',
		tech      = 'Electricity',
		buildings = {
			['Power Plant']   = {'Coal', 'Iron', 'Oil'},
			['Nuclear Plant'] = {'Uranium'},
			['Hydro Plant']   = {},
			['Solar Plant']   = {'Iron'},
		},
	},
	-- ----------------------------------------------
	{	resource  = 'Coal_Coke',
		tech      = 'Industrialization',
		buildings = {
			['Factory_Coal_Coke'] = {'Coal', 'Energy'},
		},
	},	
	-- ----------------------------------------------
	{	resource  = 'Steel',
		tech      = 'Steel',
		buildings = {
			['Factory_Steel'] = {'Coal_Coke', 'Energy', 'Iron', 'Limestone'},
		},
	},
	-- ----------------------------------------------
	{	resource  = 'Aluminum',
		tech      = 'Refining',
		buildings = {
			['Factory_Aluminum'] = {'Bauxite', 'Energy'},
		},
	},
	-- ----------------------------------------------
	{	resource  = 'Fuel',
		tech      = 'Refining',
		buildings = {
			['Factory_Fuel'] = {'Energy', 'Oil'},
		},
	},
	{	resource  = 'Plutonium',
		tech      = 'Nuclear Fission',
		buildings = {
			['Factory_Plutonium'] = {'Energy', 'Uranium'},
		},
	},
}

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __resources_terrains
-- 
--   Enter here only the natural resources that the script needs to hide
--     and the terrains they can be found.
--   Also enter here those resources that do not need extras.
--   Example: The resources 'Stones' and 'Wood' do not need extras, 
--     only terrains and infrastructures. Others will be hidden.
-- 
-- @field  <resource>   string     Resource name defined as [extra_*] 
--                                 in terrain.ruleset file
-- @field  <terrain>   array      Terrain name defined as [terrain_*] 
--                                 in terrain.ruleset file
__resources_terrains = {
	-- ----------------------------------------------
	-- Natural resources that need to be hidden
	-- ----------------------------------------------
	{   resource = 'Bauxite',
		terrains = {'Hills', 'Mountains'}
	},
	{   resource = 'Clay',
		terrains = {'Plains', 'Swamp'}
	},
	{   resource = 'Coal',
		terrains = {'Hills', 'Mountains'}
	},
	{   resource = 'Gold',
		terrains = {'Hills', 'Mountains'}
	},
	{   resource = 'Gems',
		terrains = {'Jungle'}
	},
	{   resource = 'Latex',
		terrains = {'Forest', 'Jungle'}
	},
	{   resource = 'Iron_Ore',
		terrains = {'Hills', 'Mountains'}
	},
	{   resource = 'Oil',
		terrains = {'Deep Ocean', 'Desert', 'Tundra'}
	},
	{   resource = 'Saltpeter',
		terrains = {'Desert', 'Grassland', 'Tundra'}
	},
	{   resource = 'Sulfur',
		terrains = {'Hills', 'Mountains', 'Volcano', 
		            'Erupting_Volcano_A', 'Erupting_Volcano_B'}
	},
	{   resource = 'Uranium',
		terrains = {'Hills', 'Mountains', 'Volcano', 
		            'Erupting_Volcano_A', 'Erupting_Volcano_B'}
	},	

	-- ----------------------------------------------
	-- Natural resources with NO extras:
	-- ----------------------------------------------
	{   resource = 'Stones',
		terrains = {'Mountains'}, 
	},
	{   resource = 'Wood',
		terrains = {'Forest', 'Jungle'}, 
	},
	-- ----------------------------------------------
}

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @var __specials_pct
-- 
--   Chance % to place the resource
-- 
-- @type  int (0-100)
-- 
__specials_pct = 20  -- (%)


--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __place_resources
-- 
--   Insert here all the terrains and natural resources that the script
--    should place them on.
--   In other words, here you define where to place your resources on 
--    the map.
--   Only those that have extras [extra_*] defined in terrain.ruleset
-- 
-- @field <terrain>  string      Terrain name defined as [terrain_*] 
--                                 in terrain.ruleset file
-- @field <resource> array       Resource name defined as [extra_*] 
--                                 in terrain.ruleset file
-- 
__place_resources = {
	-- ----------------------------------------------
	{	terrain   = 'Lake', 
		resources = {'Fish', 'Shellfish'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Ocean', 
		resources = {'Fish', 'Shellfish', 'Whales'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Deep Ocean', 
		resources = {'Fish', 'Whales', 'Oil'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Glacier', 
		resources = {'Ivory', 'Oil', 'Seals'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Desert', 
		resources = {'Oasis', 'Oil', 'Saltpeter'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Forest', 
		resources = {'Latex', 'Pheasant', 'Silk'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Grassland', 
		resources = {'Cattle', 'Cotton', 'Saltpeter', 'Tea'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Hills', 
		resources = {'Bauxite', 'Coal', 'Iron_Ore', 'Wine'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Jungle', 
		resources = {'Fruit', 'Gems', 'Latex', 'Sulfur'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Mountains', 
		resources = {'Bauxite', 'Gold', 'Iron_Ore', 'Sulfur', 'Uranium'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Plains', 
		resources = {'Clay', 'Corn', 'Horses', 'Wheat'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Swamp', 
		resources = {'Clay', 'Peat', 'Spice'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Tundra', 
		resources = {'Furs', 'Game', 'Saltpeter'}
	},
	-- ----------------------------------------------
	{	terrain   = 'Volcano', 
		resources = {'Iron_Ore', 'Sulfur', 'Uranium'}
	},
	-- ----------------------------------------------
}
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __units_resources
-- 
--   All units with the "SearchResources" flag and the natural 
--     resources they can reveal.
-- 
-- @field <unit>     string      Unit type with flag "SearchResources"
--                                defined as [unit_*] in units.ruleset file
-- @field <resource> array       Natural resources names to reveal
--                                defined as [extra_*] in terrain.ruleset file
-- 
__units_resources = {
	-- ----------------------------------------------
	{ 	unit      = 'Explorer',
		resources = {
			'Gems', 'Gold'
		},
	},
	-- ----------------------------------------------
	{ 	unit      = 'Miner_Limestone',
		resources = {
			'Clay', 'Gems', 'Gold', 'Limestone'
		},
	},
	-- ----------------------------------------------
	{ 	unit      = 'Miner_Iron',
		resources = {
			'Clay', 'Coal', 'Gems', 'Gold', 'Iron_Ore', 
			'Limestone'
		},
	},
	-- ----------------------------------------------
	{ 	unit      = 'Miner_Niter',
		resources = {
			'Clay', 'Coal', 'Gems', 'Gold', 'Iron_Ore', 
			'Limestone', 'Saltpeter', 'Sulfur'
		},
	},
	-- ----------------------------------------------
	{ 	unit      = 'Miner_Oil',
		resources = {
			'Oil'
		},
	},
	-- ----------------------------------------------
	{ 	unit      = 'Miner_Oil_Sea',
		resources = {
			'Oil'
		},
	},
	-- ----------------------------------------------
	{ 	unit      = 'Miner_Steel',
		resources = {
			'Bauxite', 'Clay', 'Coal', 'Gems', 'Gold', 'Iron_Ore', 
			'Limestone', 'Latex', 'Saltpeter', 'Sulfur',
		},
	},
	-- ----------------------------------------------
	{ 	unit      = 'Miner_Uranium',
		resources = {
			'Uranium'
		},
	},
}

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __buildings_resources
-- 
--   All buildings and resources the city can share with 
--     other cities that have the same building.
-- 
-- @field <building>   string       Building name defined as [building_*] 
--                                in buildings.ruleset file
-- @field <resource> array       All resources this building can share
--                                as defined in both tables in this file:
--                                * __resources_natural 
--                                * __resources_manufactured
-- 
__buildings_resources = {
	-- ----------------------------------------------
	{	building = 'Airport',
		resources = {
			'Aluminum', 'Bauxite', 'Clay', 'Coal_Coke', 
			'Coal', 'Fuel', 'Horses', 'Iron', 'Iron_Ore', 
			'Latex', 'Limestone', 'Niter', 'Oil', 'Rubber', 
			'Saltpeter', 'Steel', 'Stones', 'Sulfur', 
			'Uranium', 'Wood',
		}
	},
	-- ----------------------------------------------
	{	building = 'Harbour',
		resources = {
			'Clay', 'Coal_Coke', 'Coal', 'Horses', 'Iron', 
			'Iron_Ore', 'Limestone', 'Stones', 'Wood',
		}
	},
	-- ----------------------------------------------
}

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __units_convertion
--
--   Units which can be converted from land to sea or vice-versa.
--
-- @field from string       Unit type FROM conversion
--                           defined as [unit_*] in units.ruleset file
-- @field to   string       Unit type TO conversion
--                           defined as [unit_*] in units.ruleset file
--
__units_convertion = {
	{	from = 'Explorer',
		to   = 'Raft',
	},
	{	from = 'Raft',
		to   = 'Explorer',
	},
	{	from = 'Workers',
		to   = 'Workers_Sea',
	},
	{	from = 'Workers_Sea',
		to   = 'Workers',
	},
	{	from = 'Engineers',
		to   = 'Engineers_Sea',
	},
	{	from = 'Engineers_Sea',
		to   = 'Engineers',
	},
	{	from = 'Miner_Oil',
		to   = 'Miner_Oil_Sea',
	},
	{	from = 'Miner_Oil_Sea',
		to   = 'Miner_Oil',
	},
}

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __msgs
-- 
--   Notifications and messages 
-- 
__msgs = {	

	connection_welcome_hello = 
_([[
Welcome to I LUV Connections Ruleset. 
The task here is simple, build the appropriate infrastructure for each 
resource and connect them to the city center to unlock bonuses, units and 
improvements.
The resource must be within the city radius for the connection to be made.
More details in Help menu/About Current Ruleset
]]),

	searchresource_ready = 
	_('%s is still searching for %s. Results will be available next turn.'),

	searchresource_nothing = 
	_('%s reports that nothing was found in the vicinity of %s.'),

	searchresource_found = 
	_('BREAKING NEWS! %s reports that it found %s in the vicinity of coordinate %s.'),

	searchresource_resource = 
	_('%s %s: -------> %s.'),

	convert_unit = 
	_('Unit %s has been converted to %s in %s.'),

	connection_on = 
	_('%s connected to %s.'),

	connection_already_on = 
	_('%s is already connected to %s.'),

	connection_off = 
	_('%s has lost connection to %s.'),

	connection_export_source_success = 
	_('%s is importing %s from %s.'),

	connection_export_source_blocked = 
	_('%s cannot receive %s from %s. Source city already has this export.'),

	connection_import_blocked = 
	_('%s cannot receive %s from %s. Destination city already has this import.'),

	connection_export_blocked = 
	_('%s cannot receive %s from %s. Destination city already has this export.'),

	connection_export_incompatible = 
	_('%s cannot import %s from %s. Incompatible number of Imports.'),

	connection_import_canceled = 
	_('%s has stopped importing resource %s.'),

	connection_export_canceled = 
	_('%s has stopped exporting resource %s.'),

	connection_import_unit = 
	_('The city %s is asking for your attention! Cancel import of %s resource here?'),

	connection_import_unit_destroy = 
	_('The city of %s automatically deleted the "%s" unit.'),

	connection_import_warning = 
	_([[
***** WARNING ***** 
There is more than one city importing the resource %s. 
Choose which city will lose this connection by performing the following action in the desired city.
 --- Export resource / Break export/import ---  
The remaining units will be automatically deleted from the system. 
If you don't choose the city, the choice will be automatically randomized at the end of the turn.
]]),

	resource_searching = 
	_('%s has started searching for %s in the vicinity of %s. 1 turn without movement.'),

	resource_reveal = 
	_('BREAKING NEWS! We have found %s in the vicinity of %s.'),

	connection_no_infra = 
	_('%s reports: Target %s does not have the correct infrastructure.'),

	volcano_begin = 
	_('This volcano %s started to erupt. The citizens are concerned.'),

	volcano_end = 
	_('The volcano has calmed down at %s.'),
}


--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __grammar
--
--   Grammar control
-- 
__grammar = { 
	conjunction_and = _('and')
}

--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @table __label_resources
--
--   Resource name translation control
--
-- @key <resource>   string       Resource name as defined in one of 
--                                  the two tables:
--                                * __resources_natural;
--                                * __resources_manufactured. 
-- @value <resource> string       Resource name in current language
--
__label_resources = {

	['Aluminum']  = _('#Aluminum'),
	['Bauxite']   = _('#Bauxite'),
	['Cattle']    = _('#Cattle'),
	['Clay']      = _('#Clay'),
	['Coal']      = _('#Coal'),
	['Coal_Coke'] = _('#Coal_Coke'),
	['Corn']      = _('#Corn'),
	['Cotton']    = _('#Cotton'),
	['Energy']    = _('#Energy'),
	['Fish']      = _('#Fish'),
	['Fruit']     = _('#Fruit'),
	['Fuel']      = _('#Fuel'),
	['Furs']      = _('#Furs'),
	['Game']      = _('#Game'),
	['Gems']      = _('#Gems'),
	['Gold']      = _('#Gold'),
	['Horses']    = _('#Horses'),
	['Iron']      = _('#Iron'),
	['Iron_Ore']  = _('#Iron_Ore'),
	['Ivory']     = _('#Ivory'),
	['Latex']     = _('#Latex'),
	['Limestone'] = _('#Limestone'),
	['Niter']     = _('#Niter'),
	['Oasis']     = _('#Oasis'),
	['Oil']       = _('#Oil'),
	['Peat']      = _('#Peat'),
	['Pheasant']  = _('#Pheasant'),
	['Plutonium'] = _('#Plutonium'),
	['Rubber']    = _('#Rubber'),
	['Stones']    = _('#Stones'),
	['Saltpeter'] = _('#Saltpeter'),
	['Seals']     = _('#Seals'),
	['Shellfish'] = _('#Shellfish'),
	['Silk']      = _('#Silk'),
	['Spice']     = _('#Spice'),
	['Steel']     = _('#Steel'),
	['Sulfur']    = _('#Sulfur'),
	['Tea']       = _('#Tea'),
	['Uranium']   = _('#Uranium'),
	['Whales']    = _('#Whales'),
	['Wheat']     = _('#Wheat'),
	['Wine']      = _('#Wine'),
	['Wood']      = _('#Wood'),
}


-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
--- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- @END
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=.
-- DO NOT CHANGE AFTER THIS LINE
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_resource_invisible(nm_resource)
	local i, data_resources
	for i, data_resources in ipairs(__resources_terrains) do
		if nm_resource == data_resources.resource then
			return true
		end
	end
	return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function place_random_resource(tile, resources)
	local lenght = #resources
	local num = random(1, lenght)
	local nm_resource = resources[num]
	if nm_resource then
		local need_hide = check_resource_invisible(nm_resource)
		if need_hide then
			tile:create_extra('Mark_' .. nm_resource)
		else
			tile:create_extra(nm_resource)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function get_resources_from_terrain(place_terrain)
	local i, data_terrains
	for i, data_terrains in ipairs(__place_resources) do
		local nm_terrain = data_terrains.terrain
		if place_terrain == nm_terrain then
			return data_terrains.resources
		end
	end
	return nil
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function place_extra_resources()
	local place
	for place in whole_map_iterate() do
		if random(1, 100) <= __specials_pct then
			local place_terrain = place.terrain:rule_name()
			local resources = get_resources_from_terrain(place_terrain)
			if resources then 
				local can_place = true
				if place_terrain == 'Deep Ocean' then
					can_place = adjacent_to(place, 'Ocean')
				end
				if can_place then
					place_random_resource(place, resources)
				end
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- Check if there is certain terrain in ANY CAdjacent tile.
function adjacent_to(tile, terrain_name)
	for adj_tile in tile:circle_iterate(1) do
		if adj_tile.id ~= tile.id then
			local adj_terr = adj_tile.terrain
			local adj_name = adj_terr:rule_name()
			if adj_name == terrain_name then
				return true
			end
		end
	end
	return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function action_finished_unit_callback(action, result, actor, target)
	if actor then
		local nmAction = action:rule_name()
		if nmAction == 'Bombard Lethal' then
			run_bombard_lethal(actor, target)
		elseif nmAction == 'User Action 1' then
			run_convert_unit(actor, target)
		elseif nmAction == 'User Action 2' then
			run_export_resource(actor, target)
		elseif nmAction == 'User Action 3' then
			run_search_resources(actor, target)
		elseif nmAction == 'User Action 4' then
			run_connect_unreachable_resource(actor, target)
		end
	end
	return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_bombard_lethal(actor, target)
	actor.tile:create_extra('Bombardment')
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_convert_unit(actor, target)
	local unit_new = nil
	local moves_left = 0
	local i, data_conversion
	for i, data_conversion in ipairs(__units_convertion) do
		if has_unit_type_name(actor, data_conversion.from) then
			unit_new = find.unit_type(data_conversion.to)
			break
		end
	end
	if unit_new and unit_new:can_exist_at_tile(target) then
		notify_conversion(actor, unit_new, target)
		actor.owner:create_unit(target, unit_new, 0, actor:get_homecity(), moves_left)
		actor:kill('disbanded', actor.owner)
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_exportation(player, nm_connection, nm_importation, nm_exportation) 
	local total_exp = 0
	local total_imp = 0
	local city
	for city in player:cities_iterate() do
		local has_connection = city.tile:has_extra(nm_connection)
		local has_exportation = city.tile:has_extra(nm_exportation)
		local has_importation = city.tile:has_extra(nm_importation)

		if has_importation then
			total_imp = total_imp + 1
		end
		if has_exportation then
			total_exp = total_exp + 1
		end
	end
	return {total_exp, total_imp}
	-- if total_exp == total_imp then
	-- 	return true
	-- end
	-- return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_export_resource(actor, target)
	local city = get_city_by_tile(actor.owner, target.tile)
	if city then
		if actor.utype:has_flag('BreakExportation') then
			run_export_resource_break(actor, city)
		else
			if actor:get_homecity() ~= city then
				run_export_resource_begin(actor, city)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_resource_exist(nm_resource)
	-- local tables_resources = {__resources_natural, __resources_manufactured}
	local j, data_resources
	for j, data_resources in ipairs(__resources_natural) do
		if nm_resource == data_resources.resource then
			return true
		end
	end

	for j, data_resources in ipairs(__resources_manufactured) do
		if nm_resource == data_resources.resource then
			return true
		end
	end
	return false
end

-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function get_resource_name_from_unit(actor, name_mark)
	local unit_type = actor.utype:rule_name()
	local lenght_string = string.len(unit_type)
	local lenght_ini = string.len(name_mark)
	local nm_resource = string.sub(unit_type, (lenght_ini + 1), lenght_string)	
	if check_resource_exist(nm_resource) then
		return nm_resource
	else
		return nil
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_export_resource_break(actor, city)
	-- Units for Cancel Export and Import must have an 
	-- initial name (name_mark) of the same length.
	-- Example: 'Export_Break_' and 'Import_Break_' has the same lenght.
	local nm_resource = get_resource_name_from_unit(actor, 'Export_Break_')
	if nm_resource then
		local nm_connection = 'Connection_' .. nm_resource
		local nm_importation = nm_connection .. '_IMP'
		local nm_exportation = nm_connection .. '_EXP'
		local has_exportation = city.tile:has_extra(nm_exportation)
		local has_importation = city.tile:has_extra(nm_importation)
		if has_exportation then
			city.tile:remove_extra(nm_exportation)
		elseif has_importation then
			city.tile:remove_extra(nm_importation)
		end
		if has_exportation then
			notify_connection_export_canceled(city, nm_resource)
		elseif has_importation then
			notify_connection_import_canceled(city, nm_resource)
		end
		if has_exportation then
			local cities_ids_imp = get_cities_trading(actor.owner, nm_connection, nm_importation) 
			if #cities_ids_imp == 1 then
				local other_city_id = cities_ids_imp[1]
				local other_city = find.city(actor.owner, other_city_id)
				if other_city and other_city.tile:has_extra(nm_importation) then
					other_city.tile:remove_extra(nm_importation)
					other_city.tile:remove_extra(nm_connection)
					notify_connection_import_canceled(other_city, nm_resource)
					notify_connection_off(other_city, nm_resource)
				end
			elseif #cities_ids_imp > 1 then
				notify_connection_import_warning(actor.owner, nm_resource)
				create_units_importation(actor.owner, nm_resource)
			end
		elseif has_importation then
			-- destroy_units_importation(actor, nm_resource)
			destroy_units_importation(actor.owner, nm_resource, actor)
		end
		if has_exportation or has_importation then
			actor:kill('disbanded', actor.owner)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function destroy_units_importation(player, nm_resource, actor)
	actor = actor or nil
-- local nm_resource = get_resource_name_from_unit(actor, 'Import_Break_')
	for unit in player:units_iterate() do
		local ok = true
		if actor then 
			if actor.id ~= unit.id then
				ok = true
			else
				ok = false
			end
		end
		if ok then
			if unit.utype:has_flag('BreakExportation') then
				local unit_type = unit.utype:rule_name()
				if unit_type == 'Import_Break_' .. nm_resource then
					notify_connection_import_unit_destroy(
						unit:get_homecity(), 
						unit.utype:name_translation(), 
						nm_resource)
					unit:kill('disbanded', player)					
				end
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function create_units_importation(player, nm_resource)
	local city
	for city in player:cities_iterate() do
		local nm_importation = 'Connection_' .. nm_resource  .. '_IMP'
		local has_importation = city.tile:has_extra(nm_importation)
		if has_importation then
			local unit_new = find.unit_type('Import_Break_' .. nm_resource)
			if unit_new then
				player:create_unit(city.tile, unit_new, 0, city, -1)
				notify_connection_import_unit(city, unit_new, nm_resource)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_export_resource_begin(actor, target)
	local home = actor:get_homecity()
	if home then
		local nm_resource = get_resource_name_from_unit(actor, 'Export_')
		if nm_resource then
			local nm_connection = 'Connection_' .. nm_resource
			local nm_importation = nm_connection .. '_IMP'
			local nm_exportation = nm_connection .. '_EXP'
			
			if target.tile:has_extra(nm_importation) then
				notify_connection_import_blocked(target, home, nm_resource)
			elseif target.tile:has_extra(nm_exportation) then
				notify_connection_export_blocked(target, home, nm_resource)
			else
				local cities_ids_exp = get_cities_trading(actor.owner, nm_connection, nm_exportation) 
				local cities_ids_imp = get_cities_trading(actor.owner, nm_connection, nm_importation) 
				if #cities_ids_exp == #cities_ids_imp then 
					local has_connection = home.tile:has_extra(nm_connection)
					local has_exportation = home.tile:has_extra(nm_exportation)
					local has_importation = home.tile:has_extra(nm_importation)
					
					if has_connection and not has_exportation and not has_importation then
						if not target.tile:has_extra(nm_importation) then
							target.tile:create_extra(nm_importation)
							target.tile:create_extra(nm_connection)
						end
						if not home.tile:has_extra(nm_exportation) then
							home.tile:create_extra(nm_exportation)
						end
						actor:kill('disbanded', actor.owner)
						notify_connection_export_source_success(target, home, nm_resource)
					else
						notify_connection_export_source_blocked(target, home, nm_resource)
					end 
				else
					notify_connection_export_incompatible(target, home, nm_resource)
				end
			end
		end
	end 
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function get_cities_trading(player, nm_connection, type_trade)
	local ids = {}
	local city
	for city in player:cities_iterate() do
		local has_connection = city.tile:has_extra(nm_connection)
		local has_trade = city.tile:has_extra(type_trade)
		
		if has_connection and has_trade then
			table.insert(ids, city.id)
		end
	end
	return ids
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function get_city_by_tile(player, tile)
	local city
	for city in player:cities_iterate() do
		if city.tile == tile then
			return city
		end
	end
	return nil
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_infrastructure(player, city, next_tile, data_resources, opcao)

	opcao = opcao or 1

	local has_connection = false

	local nm_resource = data_resources.resource
	local data_infras = data_resources.infras
	local need_extra = data_resources.extra


	local has_extra = false
	if need_extra == true then
		has_extra = next_tile:has_extra(nm_resource)
	end


	if (need_extra and has_extra) or not need_extra then
		local nm_structure, data_structures
		for nm_structure, data_structures in pairs(data_infras) do
			local has_terrain = false
			local has_structure = next_tile:has_extra(nm_structure)

			if has_structure then
				local nm_terrain = next_tile.terrain:rule_name()
				local data_terrains = data_structures.terrains
				has_terrain = in_array(nm_terrain, data_terrains)
			end
			local connect1 = (has_structure and has_terrain and need_extra and has_extra)
			local connect2 = (has_structure and has_terrain and not need_extra)
			local connect3 = (has_structure and not has_terrain and need_extra and has_extra)

			if connect1 or connect2 or connect3 then
				has_connection = true
			end

			-- AI doesn't need to connect by road.
			if has_connection == true and opcao == 1 and player:is_human() then
				local need_road = data_structures.road
				if need_road == true then 
					has_connection = false
					local has_unreachable = next_tile:has_extra('Unreachable_Resource')
					if has_unreachable == true then
						has_connection = true
					elseif has_unreachable == false then
						has_connection = check_road_connected(city, next_tile, data_structures)
					end
				end     
			end
			if has_connection == true then
				break
			end
		end
	end
	return has_connection
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_connect_unreachable_resource(actor, target)
	local has_connection = false
	local city = get_city_by_tile(actor.owner, actor.tile)
	if city then
		local has_infra = false
		local i, data_resources
		for i, data_resources in ipairs(__resources_natural) do
			local has_tech = check_player_knows_tech(actor.owner, data_resources.tech)
			if has_tech == true then
				has_infra = check_infrastructure(actor.owner, city, target, data_resources, 2)
				if has_infra then
					if target:has_extra('Unreachable_Resource') then
						run_connect_unreachable_resource_off(actor, target, city, data_resources)
						break
					else
						run_connect_unreachable_resource_on(actor, target, city, data_resources)
						break
					end
				end
			end
		end
		if has_infra == false then
			notify_connection_no_infra(actor, target)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_connect_unreachable_resource_off(actor, target, city, data_resources)
	local nm_resource = data_resources.resource
	target:remove_extra('Unreachable_Resource')
	city.tile:remove_extra('Connection_' .. nm_resource)
	add_resource_connected(actor.owner, city, nm_resource, false)
	add_city_resource(actor.owner, nm_resource, city, false)
	notify_connection_off(city, nm_resource)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_connect_unreachable_resource_on(actor, target, city, data_resources)
	local nm_resource = data_resources.resource
	target:create_extra('Unreachable_Resource')
	city.tile:create_extra('Connection_' .. nm_resource)
	add_resource_connected(actor.owner, city, nm_resource, true)
	add_city_resource(actor.owner, nm_resource, city, true)
	notify_connection_on(city, nm_resource)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_search_resources(actor, target)
	local has_begin = actor.tile:has_extra('SearchResources_BEGIN')
	local has_ready = actor.tile:has_extra('SearchResources_READY')
	if not has_begin and not has_ready then
		actor.tile:create_extra('SearchResources_BEGIN')
		notify_resource_searching(actor)
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function unit_loss_messages(unit, player, reason)
	if reason == 'killed' then
		if unit.utype:has_flag('SearchResources') then
			if unit.tile:has_extra('SearchResources_READY') then
				unit.tile:remove_extra('SearchResources_READY')
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function player_phase_begin_callback(player)
	if player:is_human() then
		if game.current_turn() == 1 then
			notify_connection_welcome_hello()
		end
	end
	
	-- services for User Action 3 (Unit Flag: SearchResources)
	services_search_resources(player)

	-- Resources Connected System
	services_resources_connected(player)

	-- Reveal the resources in city radius
	services_reveal_resources(player)

	-- Stops excess import connections
	services_break_import(player, __resources_natural)
	services_break_import(player, __resources_manufactured)

end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function services_break_import(player, global_resources)
	if __cities_resources and __cities_resources[player.id] then 
		local j, data_resources
		for j, data_resources in ipairs(global_resources) do
			local nm_resource = data_resources.resource
			local nm_connection = 'Connection_' .. nm_resource
			local nm_exportation = nm_connection .. '_EXP'
			local nm_importation = nm_connection .. '_IMP'
			local cities_ids_exp = get_cities_trading(player, nm_connection, nm_exportation) 
			local cities_ids_imp = get_cities_trading(player, nm_connection, nm_importation) 
			local lenght_exp = #cities_ids_exp
			local lenght_imp = #cities_ids_imp

			if lenght_imp > 0 and lenght_exp ~= lenght_imp then 
				local total_cities
				if lenght_exp > lenght_imp then 
					total_cities = lenght_exp - lenght_imp
				elseif lenght_exp < lenght_imp then 
					total_cities = lenght_imp - lenght_exp
				end
				while total_cities > 0 do
					local random_id = random(1, #cities_ids_imp)
					local city = find.city(player, cities_ids_imp[random_id])
					if city then
						destroy_extra_importing(player, city, nm_connection, nm_importation)
						notify_connection_import_canceled(city, nm_resource)
						notify_connection_off(city, nm_resource)
						total_cities = total_cities - 1
					end
				end
				destroy_units_importation(player, nm_resource, nil)
			end
		end
	end
end

-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function destroy_extra_importing(player, city, nm_connection, nm_trade)
	if city.tile:has_extra(nm_connection) then
		city.tile:remove_extra(nm_connection)
	end
	if city.tile:has_extra(nm_trade) then
		city.tile:remove_extra(nm_trade)
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function count_factories()
	local count = 0
	local i, data
	for i, data in ipairs(__resources_manufactured) do
		-- count = count + #data.buildings
		local factories, resources
		for factories, resources in pairs(data.buildings) do
			count = count + 1
		end
	end
	return count
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function services_resources_connected(player)
	local txtNacao = player.nation:rule_name()
	if txtNacao ~= 'Animal Kingdom' then

		__cities_connections = nil
		__cities_resources = nil
		__resources_cities_ids = nil
		
		-- FIRST: Filter all the connected cities 
		cities(player)

		-- Get the resources connected INSIDE city radius 
		cities_radius(player)

		-- Share the natural resources to other cities
		cities_natural_external(player)

		local max_rounds = count_factories()
		if max_rounds < 1 then
			max_rounds = 1 
		end
		for round = 1, max_rounds do

			-- Aiports share all resources available
			cities_airports(player)

			-- Share the resources from airports
			cities_airports_external(player)
			
			-- Import/export system
			cities_exportation(player)
			
			-- Share the importing resources
			cities_exportation_external(player)
			
			-- Factories
			cities_factories(player)

			-- Connections to other cities (manufactured resources)
			cities_factories_external(player) 

		end

		-- FINAL: Mark all cities with extras 
		cities_marks(player)
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function get_resource_terrains(nm_resource)
	local i, resource_data
	for i, resource_data in ipairs(__resources_terrains) do
		if nm_resource == resource_data.resource then
			return resource_data.terrains
		end
	end
	return nil
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function find_terrains_for_resources(data_resources)
	local terrains = {} 
	local i, nm_resource
	for i, nm_resource in ipairs(data_resources) do
		local resource_terrains = get_resource_terrains(nm_resource)
		if resource_terrains ~= nil then 
			local j, terrain
			for j, terrain in ipairs(resource_terrains) do
				terrains[terrain] = true 
			end
		end
	end
	local terrain_list = {}
	for terrain in pairs(terrains) do
		table.insert(terrain_list, terrain)
	end
	return terrain_list
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function get_unit_resources(unit_type)
	local i, unit_data
	for i, unit_data in ipairs(__units_resources) do
		if unit_type == unit_data.unit then
			return unit_data.resources
		end
	end
	return nil
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function services_search_resources_mark_surroundings(unit)
	local data_resources = nil
	local unit_type = unit.utype:rule_name()
	if unit_type then
		data_resources = get_unit_resources(unit_type)
	end
	if data_resources then
		local data_terrains = find_terrains_for_resources(data_resources)
		local next_tile
		for next_tile in unit.tile:square_iterate(1) do
			local has_terrain = in_array(next_tile.terrain:rule_name(), data_terrains)
			if has_terrain then
				next_tile:create_extra('SearchResources_WORKING')
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function services_search_resources(player)
	for unit in player:units_iterate() do
		if unit.utype:has_flag('SearchResources') then
			if unit.tile:has_extra('SearchResources_BEGIN') then
				unit.tile:remove_extra('SearchResources_BEGIN')
				unit.tile:create_extra('SearchResources_READY')
				unit:movement_disallow()
				services_search_resources_mark_surroundings(unit)
				notify_searchresource_ready(unit)
			elseif unit.tile:has_extra('SearchResources_READY') then
				unit.tile:remove_extra('SearchResources_READY')
				unit:movement_allow()
				services_search_resources_1(player, unit)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function services_search_resources_1(player, unit)
	local next_tile
	local unit_type = unit.utype:rule_name()
	local data_resources = get_unit_resources(unit_type)
	if data_resources then
		local resources_found = {}
		for next_tile in unit.tile:square_iterate(1) do
		local resource_found = ''
		for _, nm_resource in ipairs(data_resources) do
			if next_tile:has_extra('Mark_' .. nm_resource) then
				resource_found = nm_resource
				next_tile:remove_extra('Mark_' .. nm_resource)
				next_tile:create_extra(nm_resource)
				break
			end
		end
		table.insert(resources_found, {tile = next_tile, resource = resource_found})
	end
	notify_resources_found(unit, resources_found)
end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- CONNECTIONS
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities(player)
	__cities_connections = nil
	__cities_connections = {}
	__cities_connections[player.id] = {}
	local city
	for city in player:cities_iterate() do
		if not __cities_connections[player.id][city.id] then
			__cities_connections[player.id][city.id] = {}
		end
		local other_city
		for other_city in player:cities_iterate() do
			if city.id ~= other_city.id then
				local path = find_path_road(city.tile, other_city.tile)
				if path then

					if not in_array(other_city.id, __cities_connections[player.id][city.id]) then
						table.insert(
							__cities_connections[player.id][city.id], other_city.id)
					end
				end
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_radius(player)
	local i, data_resources
	for i, data_resources in ipairs(__resources_natural) do
		local has_tech = check_player_knows_tech(player, data_resources.tech)
		if has_tech then
			cities_radius_1(player, data_resources)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_radius_1(player, data_resources)
	local city
	local nm_resource = data_resources.resource
	for city in player:cities_iterate() do
		local has_connection = false
		for next_tile in city.tile:circle_iterate(city:map_sq_radius()) do
			local has_owner_some = false
			local has_owner_me = false
			local nacao = next_tile.owner
			if nacao then
				for player2 in players_iterate() do
					if player.id ~= player2.id and player2.id == nacao.id then
						has_owner_some = true
						break
					end
				end
				if not has_owner_some then 
					if player.id == nacao.id then
						has_owner_me = true
					end
				end
			end
			if not has_owner_some then 
				local has_unreachable = next_tile:has_extra('Unreachable_Resource')
				if not player:is_human() or (has_unreachable or has_owner_me) then
					local has_infra = check_infrastructure(player, city, next_tile, data_resources, 1)
					if has_infra == true then
						has_connection = true
						break
					end
				end
			end
		end
		add_resource_connected(player, city, nm_resource, has_connection)
		add_city_resource(player, nm_resource, city, has_connection)
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_road_connected(city, next_tile, data_structures)
	local path = find_path_road(city.tile, next_tile)
	if path then
		return true
	end
	return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function add_resource_connected(player, city, nm_resource, has_connection)
	if not __cities_resources then
		__cities_resources = {}
	end 
	if not __cities_resources[player.id] then
		__cities_resources[player.id] = {}
	end
	__cities_resources[player.id][city.id] = __cities_resources[player.id][city.id] or {}
	__cities_resources[player.id][city.id][nm_resource] = has_connection
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function add_city_resource(player, nm_resource, city, has_connection)
	if not __resources_cities_ids then
		__resources_cities_ids = {}
	end
	if not __resources_cities_ids[player.id] then
		__resources_cities_ids[player.id] = {}
	end
	__resources_cities_ids[player.id][nm_resource] = __resources_cities_ids[player.id][nm_resource] or {}
	__resources_cities_ids[player.id][nm_resource][city.id] = has_connection
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_factories(player)
	local i, data_resources
	for i, data_resources in ipairs(__resources_manufactured) do
		local has_tech = check_player_knows_tech(player, data_resources.tech)
		if has_tech and data_resources.buildings ~= nil then
			cities_factories_1(player, data_resources)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_factories_1(player, data_resources)
	local new_resource = data_resources.resource
	local city
	for city in player:cities_iterate() do
		local nm_building, data_building
		for nm_building, data_building in pairs(data_resources.buildings) do
			local new_building = find.building_type(nm_building)
			if new_building then
				if  city:has_building(new_building) then
					local has_all_resources = false
					has_all_resources = city_has_all_resources_to_factory(player, city, data_building)

					add_resource_connected(player, city, new_resource, has_all_resources)
					add_city_resource(player, new_resource, city, has_all_resources)
				end
			end
		end           
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function city_has_all_resources_to_factory(player, city, data_building)
	local has_all_resources = true
	if  __cities_resources[player.id] ~= nil then
		if __cities_resources[player.id][city.id] ~= nil then
			local i, nm_resource
			for i, nm_resource in ipairs(data_building) do
				if __cities_resources[player.id][city.id][nm_resource] ~= nil then
					if __cities_resources[player.id][city.id][nm_resource] == false then
						has_all_resources = false
						break
					end
				end
			end
		end
		return has_all_resources
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_factories_external(player)
	if __cities_resources and  __cities_resources[player.id] then 
		local i, data_resources
		for i, data_resources in ipairs(__resources_manufactured) do
			local has_tech = check_player_knows_tech(player, data_resources.tech)
			if has_tech then
				cities_natural_external_1(player, data_resources.resource)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_natural_external(player)
	if __cities_resources and __cities_resources[player.id] then 
		local i, data_resources
		for i, data_resources in ipairs(__resources_natural) do
			local has_tech = check_player_knows_tech(player, data_resources.tech)
			if has_tech then
				cities_natural_external_1(player, data_resources.resource)
			end
		end
	end

end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_natural_external_1(player, nm_resource)
	local city
	for city in player:cities_iterate() do
		if __cities_resources[player.id][city.id] then
			find_and_mark_connections(player, nm_resource, city)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function find_and_mark_connections(player, nm_resource, city)
	local has_connection = false
	if  __cities_resources[player.id][city.id] then
		if __cities_resources[player.id][city.id][nm_resource] == nil then 
			__cities_resources[player.id][city.id][nm_resource] = false
		end
		if __cities_resources[player.id][city.id][nm_resource] ~= nil 
			and __cities_resources[player.id][city.id][nm_resource] == false then
				has_connection = find_and_mark_connections_1(player, nm_resource, city)
			end
		end
		if has_connection then
			add_resource_connected(player, city, nm_resource, true)
			add_city_resource(player, nm_resource, city, true)
		end
	end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function find_and_mark_connections_1(player, nm_resource, city)
	local has_connection = false
	if  __resources_cities_ids[player.id] ~= nil then
		if __resources_cities_ids[player.id][nm_resource] ~= nil then
			local other_city_id, has_resource
			for other_city_id, has_resource in pairs(__resources_cities_ids[player.id][nm_resource]) do
				if other_city_id ~= city.id and has_resource == true then        
					local other_city = find.city(player, other_city_id)
					if other_city then
						has_connection = find_city_connected(player, city, other_city)
						if has_connection == true then
							break
						end
					end
				end
			end
		end
	end
	return has_connection
end

-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_marks(player)
	if __cities_resources and __cities_resources[player.id] then 
		local city_id, city_resources
		for city_id, city_resources in pairs(__cities_resources[player.id]) do
			local city = find.city(player, city_id)
			if city and city_resources then
				cities_marks_1(player, city, city_resources)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_marks_1(player, city, city_resources)
	local nm_resource, has_resource
	for nm_resource, has_resource in pairs(city_resources) do
		local extraConnection = 'Connection_' .. nm_resource
		if has_resource == false then
			if city.tile:has_extra(extraConnection) then
				city.tile:remove_extra(extraConnection)
				notify_connection_off(city, nm_resource)
			end
		elseif has_resource == true then
			if not city.tile:has_extra(extraConnection) then
				city.tile:create_extra(extraConnection)
				notify_connection_on(city, nm_resource)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function get_manufactured(player, nm_resource)
	local i, data_resources
	for i, data_resources in ipairs(__resources_manufactured) do
		local resource = data_resources.resource
		if resource == nm_resource then
			return data_resources
		end
	end
	return nil
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_exportation(player)
	local city
	for city in player:cities_iterate() do
		if __cities_resources[player.id][city.id] then
			cities_exportation_1(player, city)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_exportation_1(player, city)
	local nm_resource, valor
	for nm_resource, valor in pairs(__cities_resources[player.id][city.id]) do
		local nm_importation = 'Connection_' .. nm_resource .. '_IMP'
		local has_importation = city.tile:has_extra(nm_importation)
		if has_importation then
			local has_connection = cities_exportation_2(player, city, nm_resource)
			add_resource_connected(player, city, nm_resource, has_connection)
			add_city_resource(player, nm_resource, city, has_connection)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_exportation_2(player, city, nm_resource)
	local has_connection = false
	local nm_exportation = 'Connection_' .. nm_resource .. '_EXP'
	local other_city_id, has_resource
	if __resources_cities_ids[player.id][nm_resource] ~= nil then
		for other_city_id, has_resource in pairs(__resources_cities_ids[player.id][nm_resource]) do
			if other_city_id ~= city.id and has_resource == true then
				local other_city = find.city(player, other_city_id)
				if other_city then
					local has_exportation  = other_city.tile:has_extra(nm_exportation)
					if has_exportation == true then
						has_connection = true
						break
					end
				end
			end
		end
	end
	return has_connection
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_exportation_external(player)
	if __cities_resources and __cities_resources[player.id] then 
		-- local tables_resources = {__resources_natural, __resources_manufactured}
		local j, data_resources
		for j, data_resources in ipairs(__resources_natural) do
			if check_player_knows_tech(player, data_resources.tech) then
				cities_exportation_external_1(player, data_resources.resource)
			end
		end
		for j, data_resources in ipairs(__resources_manufactured) do
			if check_player_knows_tech(player, data_resources.tech) then
				cities_exportation_external_1(player, data_resources.resource)
			end
		end

	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_exportation_external_1(player, nm_resource)
	local city
	for city in player:cities_iterate() do
		if not city.tile:has_extra('Mark_' .. nm_resource .. '_IMP') then 
			if __cities_resources[player.id][city.id] then
				find_and_mark_connections(player, nm_resource, city)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_airports(player)
	local i, data_building
	for i, data_building in ipairs(__buildings_resources) do
		local nm_building = data_building.building
		local building = find.building_type(nm_building)
		if building then
			local data_resources = data_building.resources
			local city
			for city in player:cities_iterate() do
				if city:has_building(building) then
					cities_airports_1(player, city, building, data_resources)
				end
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_airports_1(player, city, building, data_resources)
	local next_city_id, next_city_resources
	for next_city_id, next_city_resources in pairs(__cities_resources[player.id]) do
		if next_city_id ~= city.id then
			local next_city = find.city(player, next_city_id)
			if next_city and next_city:has_building(building) then
				cities_airports_2(player, city, data_resources, next_city_resources)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_airports_2(player, city, data_resources, next_city_resources)
	local nm_resource, valor
	for nm_resource, has_resource in pairs(next_city_resources) do
		local has_tech = check_player_knows_tech_by_resource(player, nm_resource)
		if has_tech then
			local can_share = in_array(nm_resource, data_resources)
			if can_share and has_resource == true then
				add_resource_connected(player, city, nm_resource, true)
				add_city_resource(player, nm_resource, city, true)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_airports_external(player)
	if __cities_resources and __cities_resources[player.id] then 
	-- local tables_resources = {__resources_natural, __resources_manufactured}
	local j, data_resources
	for j, data_resources in ipairs(__resources_natural) do
		if check_player_knows_tech(player, data_resources.tech) then
			cities_airports_external_1(player, data_resources.resource)
		end
	end
	for j, data_resources in ipairs(__resources_manufactured) do
		if check_player_knows_tech(player, data_resources.tech) then
			cities_airports_external_1(player, data_resources.resource)
		end
	end
end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_airports_external_1(player, nm_resource)
	local i, data_building
	for i, data_building in ipairs(__buildings_resources) do
		local nm_building = data_building.building
		local building = find.building_type(nm_building)
		if building then
			local city
			for city in player:cities_iterate() do
				if not city:has_building(building) then 
					if  __cities_resources[player.id][city.id] ~= nil then
						if  __cities_resources[player.id][city.id][nm_resource] == nil then 
							__cities_resources[player.id][city.id][nm_resource] = false
						end
						if  __cities_resources[player.id][city.id][nm_resource] ~= nil and 
							__cities_resources[player.id][city.id][nm_resource] == false then
								cities_airports_external_2(player, building, nm_resource, city)
							end
						end
					end
				end
			end
		end
	end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function cities_airports_external_2(player, building, nm_resource, city)
	if  __resources_cities_ids[player.id] and __resources_cities_ids[player.id][nm_resource] then
		local has_connection = false
		local other_city_id, has_resource
		for other_city_id, has_resource in pairs(__resources_cities_ids[player.id][nm_resource]) do
			if other_city_id ~= city.id and has_resource then        
				local other_city = find.city(player, other_city_id)
				if other_city and other_city:has_building(building) then
					has_connection = find_city_connected(player, city, other_city)
					if has_connection == true then
						break
					end
				end
			end
		end
		if has_connection then
			add_resource_connected(player, city, nm_resource, true)
			add_city_resource(player, nm_resource, city, true)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function find_city_connected(player, city, other_city)
	if __cities_connections[player.id] ~= nil then
		if __cities_connections[player.id][city.id] ~= nil then
			if in_array(other_city.id, __cities_connections[player.id][city.id]) then
				return true
			end
		end
	end
	return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_player_knows_tech_by_resource(player, resource)
	-- local tables_resources = {__resources_natural, __resources_manufactured}
	local j, data_resources
	for j, data_resources in ipairs(__resources_natural) do
		if resource == data_resources.resource then
			if check_player_knows_tech(player, data_resources.tech) then
				return true
			end
		end
	end
	for j, data_resources in ipairs(__resources_manufactured) do
		if resource == data_resources.resource then
			if check_player_knows_tech(player, data_resources.tech) then
				return true
			end
		end
	end
	return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_player_knows_tech(player, nm_tech)
	if nm_tech == '' then
		return true
	else 
		local tech = find.tech_type(nm_tech)
		if tech and player:knows_tech(tech) then
			return true
		end
	end
	return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function services_reveal_resources(player)
	local i, data_resources
	for i, data_resources in ipairs(__resources_terrains) do
		local nm_resource = data_resources.resource
		local has_tech = check_player_knows_tech_by_resource(player, nm_resource)
		if has_tech then
			services_reveal_resources_1(player, nm_resource)
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function services_reveal_resources_1(player, nm_resource)
	local city
	for city in player:cities_iterate() do
		if city.tile:has_extra('Reveal_' .. nm_resource) then
			break
		end
		local changed = false
		local next_tile
		for next_tile in city.tile:circle_iterate(city:map_sq_radius()) do
		if next_tile:has_extra('Mark_' .. nm_resource) then
			next_tile:remove_extra('Mark_' .. nm_resource)
			next_tile:create_extra(nm_resource)
			changed = true
		end
	end
	if changed then
		notify_resources_reveal(city, nm_resource)
		if not city.tile:has_extra('Reveal_' .. nm_resource) then
			city.tile:create_extra('Reveal_' .. nm_resource)
		end
	end
end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_any_road(next_tile)
	local hasRoad = false
	for _, nmRoad in ipairs(__roads) do
		if next_tile:has_extra(nmRoad) then
			hasRoad = true
			break
		end
	end
	return hasRoad
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function find_path_road(tileStart, tileDestination) 

	local sx = tileStart.nat_x
	local sy = tileStart.nat_y

	local dx = tileDestination.nat_x
	local dy = tileDestination.nat_y

	local start = {sy, sx}
	local destination = {dy, dx}

	local visited = {} 
	local queue = {} 
	local parent = {} 

	table.insert(queue, start)
	visited[start] = true
	while #queue > 0 do
		local current = table.remove(queue, 1)
		local currentTile = find.tile(current[2], current[1])
		if current[1] == destination[1] and current[2] == destination[2] then
			local path = {}
			while current do
				table.insert(path, 1, current)
				current = parent[current]
			end
			return path
		end
		if currentTile then

			for next_tile in currentTile:square_iterate(1) do
			local next_row = next_tile.nat_y
			local next_col = next_tile.nat_x
			local hasRoad = false
			if next_tile then
				hasRoad = check_any_road(next_tile)
			end
			if hasRoad and not visited[next_row .. ',' .. next_col] then
				table.insert(queue, {next_row, next_col})
				visited[next_row .. ',' .. next_col] = true
				parent[{next_row, next_col}] = current
			end
		end
	end
end
return nil
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function in_array(search, array)
	local i, str
	for i, str in ipairs(array) do
		if str == search then
			return true
		end
	end
	return false

end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function has_unit_type_name(unit, utype_name)
	return (unit.utype.id == find.unit_type(utype_name).id)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function has_tile_terrain_name(tile, terrain_name)
	return (tile.terrain.id == find.terrain(terrain_name).id)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=


-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- NOTIFICATIONS
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_welcome_hello(player)
	notify.all(__msgs.connection_welcome_hello)
	-- notify.event(player, nil, E.SCRIPT, __msgs.connection_welcome_hello)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function lbl_resource(nm_resource)
	if __label_resources[nm_resource] then
		return __label_resources[nm_resource]
	else
		return nm_resource
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_searchresource_finished(actor, msg)
	notify.event(actor.owner, actor.tile, E.UNIT_BECAME_VET, msg)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_searchresource_resource(actor, resources_found)
	local i, resource_data
	for i, resource_data in ipairs(resources_found) do
		local nm_resource = resource_data.resource
		local nm_terrain = resource_data.tile.terrain:name_translation()
		if nm_resource ~= '' then 
			notify.event(actor.owner, actor.tile, E.UNIT_BUILT, 
				string.format(__msgs.searchresource_resource, nm_terrain, 
					resource_data.tile:link_text(), lbl_resource(nm_resource)))
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_searchresource_ready(actor)
	local unit_type = actor.utype:rule_name()
	local data_resources = get_unit_resources(unit_type)
	if data_resources then    
		local txt = string_implode_local(data_resources)
		notify.event(actor.owner, actor.tile, E.UNIT_BECAME_VET, 
			string.format(__msgs.searchresource_ready, 
				actor.utype:name_translation(), txt, actor.tile:link_text()))
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_conversion(actor, unit_new, target)
	notify.event(actor.owner, target.tile, E.UNIT_BUILT, 
		string.format(__msgs.convert_unit, 
			actor.utype:name_translation(), 
			unit_new:name_translation(), 
			target:link_text()) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_on(city, nm_resource)
	notify.event(city.owner, city.tile, E.CITY_LOVE, 
		string.format(__msgs.connection_on,
			city:link_text(), lbl_resource(nm_resource)) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_already_on(city, nm_resource)
	notify.event(city.owner, city.tile, E.CITY_LOVE, 
		string.format(__msgs.connection_already_on,
			city:link_text(), lbl_resource(nm_resource)) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_off(city, nm_resource)
	notify.event(city.owner, city.tile, E.CITY_LOST, 
		string.format(__msgs.connection_off,
			city:link_text(), lbl_resource(nm_resource)) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_export_source_success(city_imp, city_exp, nm_resource)
	notify.event(city_imp.owner, city_imp.tile, E.CITY_LOVE, 
		string.format(__msgs.connection_export_source_success, 
			city_imp:link_text(), lbl_resource(nm_resource), city_exp:link_text()) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_export_source_blocked(city_imp, city_exp, nm_resource)
	notify.event(city_imp.owner, city_imp.tile, E.CITY_LOST, 
		string.format(__msgs.connection_export_source_blocked, 
			city_imp:link_text(), lbl_resource(nm_resource), city_exp:link_text()) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_import_blocked(city_imp, city_exp, nm_resource)
	notify.event(city_imp.owner, city_imp.tile, E.CITY_LOST, 
		string.format(__msgs.connection_import_blocked, 
			city_imp:link_text(), lbl_resource(nm_resource), city_exp:link_text()) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_export_blocked(city_imp, city_exp, nm_resource)
	notify.event(city_imp.owner, city_imp.tile, E.CITY_LOST, 
		string.format(__msgs.connection_export_blocked, 
			city_imp:link_text(), lbl_resource(nm_resource), city_exp:link_text()) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_export_incompatible(city_imp, city_exp, nm_resource)
	notify.event(city_imp.owner, city_imp.tile, E.CITY_LOST, 
		string.format(__msgs.connection_export_incompatible, 
			city_imp:link_text(), lbl_resource(nm_resource), city_exp:link_text()) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_import_canceled(city, nm_resource)
	notify.event(city.owner, city.tile, E.CITY_LOST, 
		string.format(__msgs.connection_import_canceled, 
			city:link_text(), lbl_resource(nm_resource)) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_export_canceled(city, nm_resource)
	notify.event(city.owner, city.tile, E.CITY_LOST, 
		string.format(__msgs.connection_export_canceled, 
			city:link_text(), lbl_resource(nm_resource)) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_import_unit(city, unit_new, nm_resource)
	notify.event(city.owner, city.tile, E.CARAVAN_ACTION, 
		string.format(__msgs.connection_import_unit, 
			-- city:link_text(), unit_new:name_translation(), lbl_resource(nm_resource)) )
			city:link_text(), lbl_resource(nm_resource)) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_import_unit_destroy(city, unit_name, nm_resource)
	notify.event(city.owner, city.tile, E.UNIT_BUILT, 
		string.format(__msgs.connection_import_unit_destroy, 
			city:link_text(), unit_name, lbl_resource(nm_resource)) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_import_warning(player, nm_resource)
	notify.event(player, nil, E.CARAVAN_ACTION, 
		string.format(__msgs.connection_import_warning, 
			lbl_resource(nm_resource)) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_resources_found(actor, resources_found)
	local resources = {}
	local i, resource_data
	for i, resource_data in ipairs(resources_found) do
		if resource_data.resource ~= '' then
			resources[resource_data.resource] = true
		end
	end
	local resource_list = {}
	for nm_resource in pairs(resources) do
		table.insert(resource_list, nm_resource)
	end
	local lenght = #resource_list
	local msg
	if lenght == 0 then
		msg = string.format(__msgs.searchresource_nothing,
			actor.utype:name_translation(), actor.tile:link_text())
	elseif lenght > 0 then
		local msg_found_resources = string_implode_local(resource_list)
		msg = string.format(__msgs.searchresource_found,
			actor.utype:name_translation(), msg_found_resources, actor.tile:link_text())
	end
	notify_searchresource_finished(actor, msg)
	if lenght > 0 then
		notify_searchresource_resource(actor, resources_found)
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_connection_no_infra(actor, target)
	notify.event(actor.owner, actor.tile, E.UNIT_BECAME_VET, 
		string.format(__msgs.connection_no_infra, 
			actor.utype:name_translation(), target:link_text()))
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function string_implode_local(list)
	local lenght = #list
	if lenght == 0 then
		return ''
	elseif lenght == 1 then
		return lbl_resource(list[1])
	else
		local list_local = {}
		local i, nm_resource
		for i, nm_resource in ipairs(list) do
			local txt_resource = lbl_resource(nm_resource)
			table.insert(list_local, txt_resource)
		end
		local txt = table.concat(list_local, ', ', 1, lenght - 1)
		return txt .. ' ' .. __grammar.conjunction_and .. ' ' .. list_local[lenght]
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_resource_searching(actor)
	local unit_type = actor.utype:rule_name()
	local data_resources = get_unit_resources(unit_type)
	if data_resources then    
		local txt = string_implode_local(data_resources)
		notify.event(actor.owner, actor.tile, E.UNIT_BUILT, 
			string.format(__msgs.resource_searching, 
				actor.utype:name_translation(), txt, actor.tile:link_text()) )
	end

end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_resources_reveal(city, nm_resource)
	notify.event(city.owner, city.tile, E.CITY_LOVE, 
		string.format(__msgs.resource_reveal, 
			lbl_resource(nm_resource), city:link_text()))
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function check_break_exportation(city, loser)
	city_transferred_exporting(city, loser, __resources_natural)
	city_transferred_exporting(city, loser, __resources_manufactured)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- Place Ruins at the location of the destroyed city.
function city_destroyed_callback(city, loser, destroyer)
  city.tile:create_extra('Ruins', NIL)
  check_break_exportation(city, loser)
  -- continue processing
  return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function city_transferred_callback(city, loser, winner, reason)
	check_break_exportation(city, loser)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function city_transferred_exporting(city, loser, global_resources)
	local resources = {}
	local j, data_resources
	for j, data_resources in ipairs(global_resources) do
		local nm_resource = data_resources.resource
		local nm_exportation = 'Connection_' .. nm_resource .. '_EXP'
		if city.tile:has_extra(nm_exportation) then
			city.tile:remove_extra(nm_exportation)
			table.insert(resources, nm_resource)
		end
	end
	if #resources > 0 then
		local i, nm_resource
		for i, nm_resource in ipairs(resources) do
			local nm_connection = 'Connection_' .. nm_resource
			-- local nm_exportation = nm_connection .. '_EXP'
			local nm_importation = nm_connection .. '_IMP'
			-- local cities_ids_exp = get_cities_trading(loser, nm_connection, nm_exportation) 
			local cities_ids_imp = get_cities_trading(loser, nm_connection, nm_importation) 
			if #cities_ids_imp == 1 then
				local other_city_id = cities_ids_imp[1]
				local other_city = find.city(loser, other_city_id)
				if other_city and other_city.tile:has_extra(nm_importation) then
					other_city.tile:remove_extra(nm_importation)
					other_city.tile:remove_extra(nm_connection)
					notify_connection_import_canceled(other_city, nm_resource)
					notify_connection_off(other_city, nm_resource)
				end
			elseif #cities_ids_imp > 1 then
				notify_connection_import_warning(loser, nm_resource)
				create_units_importation(loser, nm_resource)
			end
		end
	end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
-- Signals
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
signal.connect('map_generated', 'place_extra_resources')
signal.connect('unit_lost', 'unit_loss_messages')
signal.connect('player_phase_begin', 'player_phase_begin_callback')
signal.connect('action_finished_unit_extras', 'action_finished_unit_callback')
signal.connect('action_finished_unit_self',   'action_finished_unit_callback')
signal.connect('action_finished_unit_tile',   'action_finished_unit_callback')
signal.connect('action_finished_unit_unit',   'action_finished_unit_callback')
signal.connect('action_finished_unit_units',  'action_finished_unit_callback')
signal.connect('action_finished_unit_city',   'action_finished_unit_callback')
signal.connect('city_transferred', 'city_transferred_callback')
signal.connect('city_destroyed', 'city_destroyed_callback')
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
